<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Models\BankAccount;
use Illuminate\Http\Request;
use Yajra\DataTables\Facades\DataTables;
use Gate;
use Symfony\Component\HttpFoundation\Response;

class BankAccountController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        abort_if(Gate::denies('bank_account_access'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        if ($request->ajax()) {
            $query = BankAccount::all();
            $table = DataTables::of($query);
            $table->addIndexColumn();

            $table->addColumn('DT_RowId',function($row){
                return $row->id ? $row->id :'';
            });
            $table->addColumn('placeholder', '&nbsp;');
            $table->addColumn('actions', '&nbsp;');

            $table->editColumn('actions', function ($row) {
                $viewGate = 'bank_account_show';
                $editGate = 'bank_account_edit';
                $deleteGate = 'bank_account_delete';
                $statementGate = 'bank_account_statement';

                $crudRoutePart = 'bank-accounts';

                return view('partials.datatablesActions', compact(
                'statementGate',
                'viewGate',
                'editGate',
                'deleteGate',
                'crudRoutePart',
                'row'
            ));
            });

            $table->rawColumns(['actions', 'placeholder', 'header']);

            return $table->make(true);
        }

        return view('admin.bankaccount.index');
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        abort_if(Gate::denies('bank_account_create'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        return view('admin.bankaccount.create');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        abort_if(Gate::denies('bank_account_create'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $request->validate([
            'bank_name' => 'nullable',
            'account_no' => 'nullable|string|unique:bank_accounts',
            'balance' => 'nullable|numeric',
            'contact_person'=> 'nullable',
            'phone' => 'nullable|numeric',
            'remarks' => 'nullable',
        ]);

        BankAccount::create([
            'title' => $request->bank_name,
            'account_no' => $request->account_no,
            'balance' => $request->balance,
            'contact_person' => $request->contact_person,
            'phone' => $request->phone,
            'detail' => $request->remarks,
        ]);
        toastr()->success('Bank Account is created successfully!');
        return redirect()->route('admin.bank-accounts.index');
    }

    /**
     * Display the specified resource.
     *
     * @param  \App\Models\BankAccount  $bankAccount
     * @return \Illuminate\Http\Response
     */
    public function show(BankAccount $bankAccount)
    {
        abort_if(Gate::denies('bank_account_show'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        return view('admin.bankaccount.show',compact('bankAccount'));
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\Models\BankAccount  $bankAccount
     * @return \Illuminate\Http\Response
     */
    public function edit(BankAccount $bankAccount)
    {
        abort_if(Gate::denies('bank_account_edit'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        return view('admin.bankaccount.edit',compact('bankAccount'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Models\BankAccount  $bankAccount
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, BankAccount $bankAccount)
    {
        abort_if(Gate::denies('bank_account_edit'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $request->validate([
            'bank_name' => 'nullable',
            'account_no' => 'nullable|string',
            'balance' => 'nullable|numeric',
            'contact_person'=> 'nullable',
            'phone' => 'nullable|numeric',
            'remarks' => 'nullable',
        ]);

        $bankAccount->update([
            'title' => $request->bank_name,
            'account_no' => $request->account_no,
            'balance' => $request->balance,
            'contact_person' => $request->contact_person,
            'phone' => $request->phone,
            'detail' => $request->remarks,
        ]);
        toastr()->info('Bank Account is updated successfully!');
        return redirect()->route('admin.bank-accounts.index');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\Models\BankAccount  $bankAccount
     * @return \Illuminate\Http\Response
     */
    public function destroy(BankAccount $bankAccount)
    {
        abort_if(Gate::denies('bank_account_delete'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        if(isset($bankAccount->cashExpense) && count($bankAccount->cashExpense) > 0){
            toastr()->error('Bank has associated data. Please delete it first!');
            return back();
        }
        $bankAccount->delete();
        toastr()->error('Bank account has been deleted!');
        return back();
    }

    public function massDestroy(Request $request)
    {
        $request->validate([
            'ids'   => 'required|array',
        ]);
        foreach($request->ids as $id){
            $bankAccount = BankAccount::find($id);
            $this->destroy($bankAccount);
        }

        return response(null, Response::HTTP_NO_CONTENT);
    }
}
