<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Http\Requests\MassDestroyFuelInfoRequest;
use App\Http\Requests\StoreFuelInfoRequest;
use App\Http\Requests\UpdateFuelInfoRequest;
use App\Models\Fuel;
use App\Models\FuelInfo;
use App\Models\SiteRegister;
use DateTime;
use DateTimeInterface;
use Illuminate\Http\Request;
use Gate;
use Symfony\Component\HttpFoundation\Response;
use Yajra\DataTables\Facades\DataTables;

class FuelInfoController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        abort_if(Gate::denies('fuel_info_access'), Response::HTTP_FORBIDDEN, '403 Forbidden');


        if ($request->ajax()) {
            $query = FuelInfo::all();
            $table = DataTables::of($query);
            $table->addIndexColumn();

            $table->addColumn('DT_RowId', function ($row) {
                return $row->id ? $row->id : '';
            });

            $table->addColumn('placeholder', '&nbsp;');
            $table->addColumn('actions', '&nbsp;');

            $table->editColumn('actions', function ($row) {
                $viewGate = 'fuel_info_show';
                $editGate = 'fuel_info_edit';
                $deleteGate = 'fuel_info_delete';
                $crudRoutePart = 'fuel-infos';

                return view('partials.datatablesActions', compact(
                'viewGate',
                'editGate',
                'deleteGate',
                'crudRoutePart',
                'row'
            ));
            });

            $table->editColumn('fuel', function ($row) {
                return $row->fuel->fuel_name ? $row->fuel->fuel_name : '';
            });

            $table->editColumn('branch', function ($row) {
                return $row->siteRegister ? $row->siteRegister->name : '';
            });

            $table->editColumn('cost_price', function ($row) {
                return $row->cost_price ? $row->cost_price : 0;
            });

            $table->editColumn('selling_price', function ($row) {
                return $row->selling_price ? $row->selling_price : 0;
            });

            $table->editColumn('effective_from', function ($row) {
                return $row->effective_from ? date('d/m/Y',strtotime($row->effective_from)) : '';
            });

            $table->editColumn('effective_to', function ($row) {
                return $row->effective_to ? date('d/m/Y',strtotime($row->effective_to)) : '';
            });

            $table->addColumn('status', function ($row) {
                return $row->status ? '<span class="badge bg-inverse-success">Active</span>' : '<span class="badge bg-inverse-danger">Inactive</span>';
            });


            $table->rawColumns(['actions', 'placeholder', 'header','status']);

            return $table->make(true);
        }

        return view('admin.fuelInfo.index');
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        abort_if(Gate::denies('fuel_info_create'), Response::HTTP_FORBIDDEN, '403 Forbidden');
        $fuels = Fuel::all()->pluck('fuel_name', 'id')->prepend(trans('global.pleaseSelect'), '');
        $branches = SiteRegister::all()->pluck('name', 'id')->prepend(trans('global.pleaseSelect'), '');

        return view('admin.fuelInfo.create',compact('fuels','branches'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(StoreFuelInfoRequest $request)
    {
        if($request->ajax())
        {
            // Retrive all data from request
            foreach ($request->effective_from as $key => $value)
            {
                // $date1 = strtr($request->effective_from[$key], '/', '-');
                $eff_from = date('Y-m-d', strtotime($request->effective_from[$key]));

                // Check request has any status
                if ($request->has('status')) {

                    foreach ($request->status as $id => $val) {
                        if($key == $id)
                        {
                            // dump($key , $id);
                            // Check Active fuel info with branch and fuel
                            $lastFuelInfo = FuelInfo::where('branch_id',$request->branch[$key])
                                                    ->where('fuel_id',$request->fuel[$key])
                                                    ->where('status',1)
                                                    ->first();
                            if($lastFuelInfo != null)
                            {
                                FuelInfo::where('branch_id',$request->branch[$key])
                                            ->where('fuel_id',$request->fuel[$key])
                                            ->where('status',1)
                                            // ->each
                                            ->update([
                                                'effective_to' => $eff_from,
                                                'status' => 0,
                                            ]);

                                $fuelInfo = FuelInfo::create([
                                    'fuel_id' => $request->fuel[$key],
                                    'branch_id' =>$request->branch[$key],
                                    'cost_price' => $request->cost_price[$key],
                                    'selling_price' => $request->selling_price[$key],
                                    'effective_from' => $eff_from,
                                    'status' => $val,
                                ]);
                            }
                            else{
                                $fuelInfo = FuelInfo::create([
                                    'fuel_id' => $request->fuel[$key],
                                    'branch_id' =>$request->branch[$key],
                                    'cost_price' => $request->cost_price[$key],
                                    'selling_price' => $request->selling_price[$key],
                                    'effective_from' => $eff_from,
                                    'status' => $val,
                                ]);
                            }
                        }
                    }

                }
                else
                {
                    $fuelInfo = FuelInfo::create([
                        'fuel_id' => $request->fuel[$key],
                        'branch_id' =>$request->branch[$key],
                        'cost_price' => $request->cost_price[$key],
                        'selling_price' => $request->selling_price[$key],
                        'effective_from' => $eff_from,
                        'status' => 0,
                    ]);
                }


            }

            return response()->json(['success'=>true,'message'=>'Fuel information added successfully!','url'=> route('admin.fuel-infos.index')]);
        }
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show(FuelInfo $fuelInfo)
    {
        abort_if(Gate::denies('fuel_info_show'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        return view('admin.fuelInfo.show', compact('fuelInfo'));
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit(FuelInfo $fuelInfo)
    {
        abort_if(Gate::denies('fuel_info_edit'), Response::HTTP_FORBIDDEN, '403 Forbidden');
        $site = SiteRegister::with('fuels')->where('id',$fuelInfo->branch_id)->first();
        $fuels = $site != null ? $site->fuels->unique()->pluck('fuel_name','id') : null;
        $branches = SiteRegister::all();

        return view('admin.fuelInfo.edit', compact('fuelInfo','fuels','branches'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(UpdateFuelInfoRequest $request, $id)
    {
        $fuelInfo = FuelInfo::find($id);

        if($fuelInfo == null)
        {
            return back();
        }

        $date1 = strtr($request->input('effective_from'), '/', '-');
        $eff_from = date('Y-m-d', strtotime($date1));

        // Check status is checked
        if($request->has('status'))
        {
            // Find active status fuel info using branch and fuel
            $lastFuelInfo = FuelInfo::where('branch_id',$request->input('branch_id'))
                                    ->where('fuel_id',$request->input('fuel_id'))
                                    ->where('status',1)
                                    ->first();

            if($lastFuelInfo != null)
            {
                // Update last active fuel info
                FuelInfo::where('branch_id',$request->branch_id)
                    ->where('fuel_id',$request->fuel_id)
                    ->where('status',1)
                    ->update([
                        'effective_to' => $eff_from,
                        'status' => 0,
                    ]);
                // Update current fuel info
                $fuelInfo = FuelInfo::where('id',$id)
                            ->update([
                                        'fuel_id' => $request->input('fuel_id'),
                                        'branch_id' =>$request->input('branch_id'),
                                        'cost_price' => $request->input('cost_price'),
                                        'selling_price' => $request->input('selling_price'),
                                        'effective_from' => $eff_from,
                                        'status' => $request->input('status'),
                                    ]);
            }
            else
            {
                $fuelInfo = FuelInfo::where('id',$id)
                            ->update([
                                        'fuel_id' => $request->input('fuel_id'),
                                        'branch_id' =>$request->input('branch_id'),
                                        'cost_price' => $request->input('cost_price'),
                                        'selling_price' => $request->input('selling_price'),
                                        'effective_from' => $eff_from,
                                        'status' => $request->input('status'),
                                    ]);
            }

        }
        else
        {
            $fuelInfo = FuelInfo::where('id',$id)
                        ->update([
                                    'fuel_id' => $request->input('fuel_id'),
                                    'branch_id' =>$request->input('branch_id'),
                                    'cost_price' => $request->input('cost_price'),
                                    'selling_price' => $request->input('selling_price'),
                                    'effective_from' => $eff_from,
                                    'status' => 0,
                                ]);
        }


        toastr()->info('Fuel Info has been updated successfully!');
        return redirect()->route('admin.fuel-infos.index');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy(FuelInfo $fuelInfo)
    {
        abort_if(Gate::denies('fuel_info_delete'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $fuelInfo->delete();
        toastr()->error('Fuel Info has been deleted!');
        return back();
    }
    public function massDestroy(MassDestroyFuelInfoRequest $request)
    {
        FuelInfo::whereIn('id', request('ids'))->delete();

        toastr()->error('Fuel has been deleted!');
        return response(null, Response::HTTP_NO_CONTENT);
    }

}
