<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Http\Requests\MassDestroyPaymentAdviceRequest;
use App\Http\Requests\StorePaymentAdviceRequest;
use App\Http\Requests\UpdatePaymentAdviceRequest;
use App\Models\Fuel;
use App\Models\PaymentAdvice;
use Gate;
use App\Models\SiteRegister;
use App\Models\FuelInfo;
use Illuminate\Http\Request;
use Symfony\Component\HttpFoundation\Response;
use Yajra\DataTables\Facades\DataTables;

class PaymentAdviceController extends Controller
{
    public function __construct()
    {
        $this->middleware(function ($request, $next) {
            $this->branch_id = session()->get('branch_id');
            return $next($request);
        });

    }
    public function index(Request $request)
    {
        abort_if(Gate::denies('payment_advice_access'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        if ($request->ajax()) {
            $start =  date('Y-m-d', strtotime(strtr($request->min, '/', '-')));
            $end = date('Y-m-d', strtotime(strtr($request->max, '/', '-')));
            $query = PaymentAdvice::query()->where('branch_id',$this->branch_id);
            if($request->min != null || $request->max != null ){
                $query = $query->whereDate('date','>=',date('Y-m-d',strtotime($start)))->whereDate('date','<=',date('Y-m-d',strtotime($end)));
            }

            $table = DataTables::of($query);
            $table->addIndexColumn();

            $table->addColumn('DT_RowId',function($row){
                return $row->id ? $row->id : '';
            });
            $table->addColumn('placeholder', '&nbsp;');
            $table->addColumn('actions', '&nbsp;');

            $table->editColumn('actions', function ($row) {
                $viewGate = 'payment_advice_show';
                $editGate = 'payment_advice_edit';
                $deleteGate = 'payment_advice_delete';
                $crudRoutePart = 'payment-advices';

                return view('partials.datatablesActions', compact(
                'viewGate',
                'editGate',
                'deleteGate',
                'crudRoutePart',
                'row'
            ));
            });

            $table->editColumn('date', function ($row) {
                return $row->date ? date('d/m/Y',strtotime($row->date)) : '';
            });

            $table->editColumn('invoice',function($row){
                return $row->invoice ? $row->invoice : '';
            });

            $table->editColumn('d_note_no', function ($row) {
                return $row->d_note_no ? $row->d_note_no : '';
            });
            $table->addColumn('product', function ($row) {
                return $row->fuel->fuel_name ? $row->fuel->fuel_name : '';
            });

            $table->editColumn('r_quantity', function ($row) {
                return $row->r_quantity ? $row->r_quantity : '';
            });
            $table->editColumn('price', function ($row) {
                return $row->price ? $row->price : '';
            });
            $table->editColumn('amount', function ($row) {
                return $row->amount ? $row->amount : '';
            });
            $table->rawColumns(['actions', 'placeholder', 'paymentAdvice']);

            return $table->make(true);
        }

        return view('admin.paymentAdvices.index');
    }

    public function create()
    {
        abort_if(Gate::denies('payment_advice_create'), Response::HTTP_FORBIDDEN, '403 Forbidden');
        $site = SiteRegister::with('fuels')->where('id',$this->branch_id)->first();
        $fuels = collect();
        if(!empty($site)){
            $fuels = $site->fuels->unique()->pluck('fuel_name','id')->prepend(trans('global.pleaseSelect'),'');
        }
        return view('admin.paymentAdvices.create',compact('fuels'));
    }

    public function store(StorePaymentAdviceRequest $request)
    {
        if ($request->ajax()) {
            $req_date = strtr($request->date,'/','-');
            $date = date('Y-m-d',strtotime($req_date));
            foreach ($request->invoice_no as $key => $value) {
                $amount = null;
                $price = null;
                $r_quantity = null;
                $d_note_no = null;
                $fuel_id = null;

                if ($request->has('d_note_no') && $request->d_note_no != null) {
                    foreach ($request->d_note_no as $id => $val) {
                        if ($key == $id && $val != null) {
                            $d_note_no = $request->d_note_no[$key];
                        }
                    }
                }

                if ($request->has('product') && $request->product != null) {
                    foreach ($request->product as $id => $val) {
                        if ($key == $id && $val != null) {
                            $fuel_id = $request->product[$key];
                        }
                    }
                }

                $fuelprices = FuelInfo::where('branch_id',$this->branch_id)->whereDate('effective_from','<=',$date)->whereNull('effective_to')->orWhereDate('effective_to','>',$date)->where('fuel_id',$fuel_id)->first();

                PaymentAdvice::create([
                    'date' => $date,
                    'invoice' => $request->invoice_no[$key],
                    'd_note_no' => $d_note_no,
                    'fuel_id' => $fuel_id,
                    'r_quantity' => $request->r_quantity[$key] ,
                    'price' => isset($fuelprices->cost_price) ? $fuelprices->cost_price : null ,
                    'amount' => isset($fuelprices->cost_price) ? $fuelprices->cost_price * $request->r_quantity[$key] : '0',
                    'branch_id' => $this->branch_id,
                ]);
            }
            return response()->json(['success'=> true,'message' => 'Payment advice created successfully!','url' => route('admin.payment-advices.index')]);
        }
    }

    public function edit(PaymentAdvice $paymentAdvice)
    {
        abort_if(Gate::denies('payment_advice_edit'), Response::HTTP_FORBIDDEN, '403 Forbidden');
        $site = SiteRegister::with('fuels')->where('id',$this->branch_id)->first();
        $fuels = collect();
        if(!empty($site)){
            $fuels = $site->fuels->unique()->pluck('fuel_name','id')->prepend(trans('global.pleaseSelect'),'');
        }

        return view('admin.paymentAdvices.edit', compact('paymentAdvice','fuels'));
    }

    public function update(UpdatePaymentAdviceRequest $request, PaymentAdvice $paymentAdvice)
    {
        $req_date = strtr($request->date,'/','-');
        $date = date('Y-m-d',strtotime($req_date));
        $fuelprices = FuelInfo::where('branch_id',$this->branch_id)->whereDate('effective_from','<=',$date)->whereNull('effective_to')->orWhereDate('effective_to','>',$date)->where('fuel_id',$request->product)->first();
        $amount = 0;

        $paymentAdvice->update([
            'date' => $date,
            'invoice' => $request->invoice_no,
            'd_note_no' => $request->d_note_no,
            'fuel_id' => $request->product,
            'r_quantity' => $request->r_quantity ? $request->r_quantity : null,
            'price' => isset($fuelprices->cost_price) ? $fuelprices->cost_price : null,
            'amount' => isset($fuelprices->cost_price) ? $fuelprices->cost_price * $request->r_quantity : '0',
            'branch_id' => $this->branch_id,
        ]);

        toastr()->info('Payment advice has been updated successfully!');
        return redirect()->route('admin.payment-advices.index');
    }

    public function show(PaymentAdvice $paymentAdvice)
    {
        abort_if(Gate::denies('payment_advice_show'), Response::HTTP_FORBIDDEN, '403 Forbidden');
        return view('admin.paymentAdvices.show', compact('paymentAdvice'));
    }

    public function destroy(PaymentAdvice $paymentAdvice)
    {
        abort_if(Gate::denies('payment_advice_delete'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $paymentAdvice->forceDelete();
        toastr()->error('Payment advice has been deleted!');
        return back();
    }

    public function massDestroy(MassDestroyPaymentAdviceRequest $request)
    {
        PaymentAdvice::whereIn('id', request('ids'))->forceDelete();
        toastr()->error('Payment advice has been deleted!');
        return response(null, Response::HTTP_NO_CONTENT);
    }
}
