<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Http\Requests\MassDestroySiteRegisterRequest;
use App\Http\Requests\StoreSiteRegisterRequest;
use App\Http\Requests\UpdateSiteRegisterRequest;
use App\Models\AllCity;
use App\Models\AllCountry;
use App\Models\SitePump;
use App\Models\AllState;
use App\Models\Country;
use App\Models\SiteRegister;
use App\Models\Type;
use App\Models\Fuel;
use App\Models\Pump;
use Illuminate\Http\Request;
use Gate;
use Illuminate\Support\Str;
use NunoMaduro\Collision\Adapters\Phpunit\State;
use Symfony\Component\HttpFoundation\Response;
use Yajra\DataTables\Facades\DataTables;

class SiteRegisterController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        abort_if(Gate::denies('fuel_site_access'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        if ($request->ajax()) {
            $query = SiteRegister::all();
            $table = DataTables::of($query);
            $table->addIndexColumn();

            $table->addColumn('DT_RowId',function($row){
                return $row->id ? $row->id : '';
            });
            $table->addColumn('placeholder', '&nbsp;');
            $table->addColumn('actions', '&nbsp;');

            $table->editColumn('actions', function ($row) {
                $viewGate = 'fuel_site_show';
                $editGate = 'fuel_site_edit';
                $deleteGate = 'fuel_site_delete';
                $crudRoutePart = 'site-registers';

                return view('partials.datatablesActions', compact(
                'viewGate',
                'editGate',
                'deleteGate',
                'crudRoutePart',
                'row'
            ));
            });

            $table->editColumn('name', function ($row) {
                return $row->name ? $row->name : '';
            });

            $table->editColumn('type', function ($row) {
                return $row->type->name ? $row->type->name : '';
            });

            $table->editColumn('city', function ($row) {
                return AllCity::find($row->city_id)->name;
            });

            $table->rawColumns(['actions', 'placeholder', 'Site']);

            return $table->make(true);
        }
        return view('admin.siteRegisters.index');
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        abort_if(Gate::denies('fuel_site_create'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $countries = Country::all();
        $types = Type::all();
        $pumps = Pump::all();
        $fuels = Fuel::all();

        return view('admin.siteRegisters.create',compact('countries','types','pumps','fuels'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(StoreSiteRegisterRequest $request)
    {
        $siteReg = SiteRegister::create([
            'name' => $request->input('name'),
            'slug' => Str::slug($request->input('name'),'-'),
            'type_id' => $request->input('type_id'),
            'country' => $request->input('country'),
            'state_id' => $request->input('state_id'),
            'city_id' => $request->input('city_id'),
            'remarks' => $request->input('remarks'),
        ]);
        $pumps = $request->pump_ids;
        $fuels = $request->fuel_ids;
        $data = [];
        foreach ($pumps as $key => $id){
            $siteReg->pumps()->attach([$pumps[$key] => ['fuel_id' => $fuels[$key]]]);
        }

        if($request->ajax()){
            return response()->json('Site added successfully!');
        }
        return redirect()->route('admin.site-registers.index');
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        abort_if(Gate::denies('fuel_site_show'), Response::HTTP_FORBIDDEN, '403 Forbidden');
        $siteReg = SiteRegister::find($id);

        return view('admin.siteRegisters.show', compact('siteReg'));
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        abort_if(Gate::denies('fuel_site_edit'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $siteReg = SiteRegister::find($id);
        $pumps = Pump::all();
        $fuels = Fuel::all();
        $sitepumps = SitePump::where('site_register_id',$id)->get();
        $types =  Type::all();
        $countries = Country::all();
        $country = AllCountry::where('iso3',$siteReg->country)->first();
        $states = AllState::where('country_id',$country->id)->get();
        $cities = AllCity::where('state_id',$siteReg->state_id)->get();
        return view('admin.siteRegisters.edit', compact('siteReg','types','countries','pumps','fuels','sitepumps','states','cities'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(UpdateSiteRegisterRequest $request, SiteRegister $siteRegister)
    {
        $siteRegister->update($request->except('fuel_ids','pump_ids'));

        $pumps = $request->pump_ids;
        $fuels = $request->fuel_ids;
        $data = [];
        $siteRegister->pumps()->detach();
        if(count($pumps) > 0 & count($fuels) > 0 && $pumps[0] != null && $fuels[0] != null){
            foreach ($pumps as $key => $id){
                $siteRegister->pumps()->attach([$pumps[$key] => ['fuel_id' => $fuels[$key]]]);
            }
        }
        if($request->ajax()){
            return response()->json('Site updated successfully!');
        }
        return redirect()->route('admin.site-registers.index');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy(SiteRegister $siteRegister)
    {
        abort_if(Gate::denies('fuel_site_delete'), Response::HTTP_FORBIDDEN, '403 Forbidden');
        if($siteRegister->id == 1){
            toastr()->error('You cant delete this site as it has associated users and data!');
            return back()->withErrors('You cant delete this site as it has associated users and data');
        }
        if(isset($siteRegister->drivewaysheet) && count($siteRegister->drivewaysheet) > 0){
            toastr()->error('Site has associated data. Please delete it first!');
            return back();
        }
        $siteRegister->forceDelete();
        toastr()->error('Site has been deleted!');
        return back();
    }


    public function massDestroy(Request $request)
    {
        abort_if(Gate::denies('fuel_site_delete'), Response::HTTP_FORBIDDEN, '403 Forbidden');
        $request->validate([
            'ids'   => 'required|array',
        ]);

        foreach($request->ids as $id){
            $siteRegister = SiteRegister::find($id);
            $this->destroy($siteRegister);
        }
        // toastr()->error('Site has been deleted!');
        return response(null, Response::HTTP_NO_CONTENT);
    }

    public function states($id)
    {
        $iso3 = $id;
        $country = AllCountry::where('iso3',$iso3)->first();
        $states = AllState::where('country_id',$country->id)->get();

        return json_encode($states);
    }

    public function cities($id)
    {

       $cities = AllCity::where('state_id',$id)->get();

        return json_encode($cities);
    }


    public function fetch_fuels($id)
    {
        $site = SiteRegister::with('fuels')->where('id',$id)->first();
        $fuels = collect();
        if(!empty($site)){
            $fuels = $site->fuels->unique()->pluck('fuel_name','id');
        }
        return json_encode($fuels);
    }


    public function fetch_pumps($id)
    {
        $site = SiteRegister::with('pumps')->where('id',$id)->first();
        $pumps = collect();
        if(!empty($site)){
            $pumps = $site->pumps->unique()->pluck('pump','id');
        }
        return json_encode($pumps);
    }
}
