<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Http\Requests\StoreSpeedPointRequest;
use App\Http\Requests\UpdateSpeedPointRequest;
use App\Http\Requests\MassDestroySpeedPointRequest;
use App\Imports\SpeedPointImport;
use App\Models\Machine;
use App\Models\ShiftNo;
use App\Models\SpeedPoint;
use Illuminate\Http\Request;
use Symfony\Component\HttpFoundation\Response;
use Gate;
use App\Models\BankStatement;
use App\Models\BankAccount;
use DB;
use Excel;
use Yajra\DataTables\Facades\DataTables;

class SpeedPointController extends Controller
{
    public function __construct()
    {
        $this->middleware(function ($request, $next) {
            $this->branch_id = session()->get('branch_id');
            return $next($request);
        });

    }
    /**
     * Display a listing of the Speed Point.
     *
     * @return Symfony\Component\HttpFoundation\Response
     */
    public function index(Request $request)
    {
        abort_if(Gate::denies('speed_point_access'), Response::HTTP_FORBIDDEN, '403 Forbidden');
        $bankdata =  BankStatement::where('description','LIKE',"%Speedpoint%")->select('id', 'status', 'bank_account_id', DB::raw("CONCAT(description,' [id: ',id ,',Amount: ',amount,',Date:  ',date,']') AS name"))->get();
        if ($request->ajax()) {
            $start =  date('Y-m-d', strtotime(strtr($request->min, '/', '-')));
            $end = date('Y-m-d', strtotime(strtr($request->max, '/', '-')));
            $query = SpeedPoint::query()->where('branch_id',$this->branch_id)->select('date','batch_no','reconcilation_type','status','bank_statement_id','bank_account_id',\DB::raw("SUM(amount) as total"))->groupBy(['reconcilation_type','date','batch_no']);
            if($request->min != null || $request->max != null ){
                $query = $query->whereDate('date','>=',date('Y-m-d',strtotime($start)))->whereDate('date','<=',date('Y-m-d',strtotime($end)));
            }
            $table = Datatables::of($query);
            $table->addIndexColumn();

            $table->addColumn('placeholder', '&nbsp;');
            $table->addColumn('actions', '&nbsp;');

            $table->editColumn('actions', function ($row) {
                if($row->batch_no != null){
                    $button = '';
                    $show = '<a href="'.route('admin.speed-points.show',[$row->reconcilation_type, $row->date,$row->batch_no]).'" class="btn btn-primary btn-xs"><i class="la la-eye"></i></a> ';

                    $edit = '<a href="'.route('admin.speed-points.edit',[$row->reconcilation_type, $row->date,$row->batch_no]).'" class="btn btn-info btn-xs"><i class="la la-edit"></i></a> ';

                    $delete = '<a href="#" data-date="'.$row->date.'" data-machine="'.$row->reconcilation_type.'" data-batch="'.$row->batch_no.'" class="delete-speed btn btn-danger btn-xs"><i class="la la-trash"></i></a> ';

                    if (Gate::allows('speed_point_show')) {
                       $button = $show;
                    }
                    if (Gate::allows('speed_point_edit')) {
                        $button .= $edit;
                    }
                    if (Gate::allows('speed_point_delete')) {
                        $button .= $delete;
                    }
                    return $button;
                }
            });
            $table->addColumn('reconcilation_type', function ($row) {
               $data  = isset($row->machine) ? $row->machine->title : '';
               $data .= $row->status == 1 ? '<br><span class="badge bg-inverse-success">Approved</span>' : '<br><span class="badge bg-inverse-danger">Not Approved</span>';
               return $data;
            });
            $table->addColumn('date', function ($row) {
                return $row->date ? date('d/m/Y',strtotime($row->date)) : '';
            });

            $table->addColumn('batch_no', function ($row) {
                $data = $row->batch_no ? $row->batch_no : '';
                $data .= '<input type="hidden" value='.$row->reconcilation_type.' class="speedmachine">';
                $data .= '<input type="hidden" value='.$row->batch_no.' class="speedbatch">';
                return $data;
            });

            $table->editColumn('bank_account_id',function($row){
                return $row->bankAccount ? $row->bankAccount->title : '';
            });
            $table->addColumn('total',function($row){
                return $row->total;
            });
            if(Gate::allows('deposit_bank_statement')){
                $table->addColumn('bank_statement_id', function ($row) use ($bankdata) {
                    // return $row->bank_statement_id;
                    $bankstatement = $bankdata->where('bank_account_id',$row->bank_account_id);
                    $options = '<option value="">Please Select</option>';
                    // here we prepare the options
                    foreach ($bankstatement as $bank) {
                        if($bank->status == "0"){
                            $options .= '<option value="'.$bank->id.'">'.$bank->name.'</option>';
                        }
                        if($row->bank_statement_id == $bank->id){
                            $options .= '<option value="'.$bank->id.'" selected>'.$bank->name.'</option>';
                        }
                    }

                    $return =
                        '<div class="form-group">
                            <select class="form-control select-bank">' . $options . '</select>
                        </div>';

                    return $return;
                });
            }

            $table->rawColumns(['actions', 'placeholder', 'total','reconcilation_type','batch_no','date','bank_statement_id']);

            return $table->make(true);
        }

        return view('admin.speedPoints.index');
    }

    /**
     * Show the form for creating
     *  a new Speed point.
     *
     * @return Symfony\Component\HttpFoundation\Response
     */
    public function create()
    {
        abort_if(Gate::denies('speed_point_create'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $bankAccount = BankAccount::select('id', DB::raw("CONCAT(title,' ',IFNULL(account_no,'')) AS title1"))->get()->pluck('title1','id')->prepend(trans('global.pleaseSelect'),'');
        $shift = ShiftNo::where('branch_id',$this->branch_id)->where('status',1)->get()->pluck('title','id')->prepend(trans('global.pleaseSelect'),'');
        $machines = Machine::all()->pluck('title','id')->prepend(trans('global.pleaseSelect'),'');
        return view('admin.speedPoints.create',compact('shift','machines','bankAccount'));
    }

    /**
     * Store a newly created Speed point in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return Symfony\Component\HttpFoundation\Response
     */
    public function store(StoreSpeedPointRequest $request)
    {
        if ($request->ajax()) {
            $shift = null;
            $req_date = strtr($request->date,'/','-');
            $date = date('Y-m-d',strtotime($req_date));
            foreach ($request->seq_no as $key => $value) {
                // check request has shift
                if ($request->has('shift')) {
                    foreach ($request->shift as $id => $val) {
                        if ($key == $id && $val != null) {
                            $shift = $request->shift[$key];
                        }
                    }
                }
                // Store data
                SpeedPoint::create([
                    'reconcilation_type' => $request->reconcilation_type,
                    'date' => $date,
                    'shift_no' => $shift,
                    'batch_no' => $request->batch_no,
                    'seq_no' => $request->seq_no[$key],
                    'amount' => $request->amount[$key],
                    'branch_id' => $this->branch_id,
                    'bank_account_id' => $request->bank_account,
                ]);
            }
            return response()->json(['success'=>true,'message'=>'Speed points added successfully!','url'=> route('admin.speed-points.index')]);
        }
    }

    /**
     * Display the specified Speed Point.
     *
     * @param  int  $id
     * @return Symfony\Component\HttpFoundation\Response
     */
    public function show($machine, $date,$batch)
    {
        abort_if(Gate::denies('speed_point_show'), Response::HTTP_FORBIDDEN, '403 Forbidden');
        $speedPoint = SpeedPoint::where('branch_id',$this->branch_id)->where('reconcilation_type',$machine)->where('date',$date)->where('batch_no',$batch)->get();

        return view('admin.speedPoints.show', compact('speedPoint','date','batch','machine'));
    }

    /**
     * Show the form for editing the specified
     * Speed point.
     *
     * @param  int  $id
     * @return Symfony\Component\HttpFoundation\Response
     */
    public function edit($machine,$date,$batch)
    {
        abort_if(Gate::denies('speed_point_edit'), Response::HTTP_FORBIDDEN, '403 Forbidden');
        $speedpoint = SpeedPoint::where('branch_id',$this->branch_id)->where('reconcilation_type',$machine)->where('date',$date)->where('batch_no',$batch)->get();
        $shift = ShiftNo::where('branch_id',$this->branch_id)->where('status',1)->get()->pluck('title','id')->prepend(trans('global.pleaseSelect'),'');
        $bankAccount = BankAccount::select('id', DB::raw("CONCAT(title,' ',IFNULL(account_no,'')) AS title1"))->get()->pluck('title1','id')->prepend(trans('global.pleaseSelect'),'');
        $machines = Machine::all()->pluck('title','id')->prepend(trans('global.pleaseSelect'),'');
        return view('admin.speedPoints.edit', compact('speedpoint','machines','shift','batch','date','bankAccount','machine'));
    }

    /**
     * Update the specified Speed Point in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return Symfony\Component\HttpFoundation\Response
     */
    public function update(UpdateSpeedPointRequest $request)
    {
        if ($request->ajax()) {
            $shift = null;
            $req_date = strtr($request->date,'/','-');
            $date = date('Y-m-d',strtotime($req_date));
            foreach($request->speed_id as $key => $value) {
                $speed = SpeedPoint::find($value);
                if(!empty($speed)){
                    // Update data
                    $speed->update([
                        'reconcilation_type' => $request->reconcilation_type,
                        'date' => $date,
                        'shift_no' => $request->shift[$key],
                        'batch_no' => $request->batch_no,
                        'seq_no' => $request->seq_no[$key],
                        'amount' => $request->amount[$key],
                        'branch_id' => $this->branch_id,
                        'bank_account_id' => $request->bank_account,
                    ]);
                }
            }
            //update newly added row
            foreach($request->seq_no_new as $key => $value) {
                if ($request->amount_new[$key] != null && $request->seq_no_new[$key] != null) {
                    // $request->validate([
                    //     'seq_no' => 'unique:speed_points,seq_no'
                    // ]);
                    SpeedPoint::create([
                        'reconcilation_type' => $request->reconcilation_type,
                        'date' => $date,
                        'shift_no' => $request->shift_new[$key],
                        'batch_no' => $request->batch_no,
                        'seq_no' => $request->seq_no_new[$key],
                        'amount' => $request->amount_new[$key],
                        'branch_id' => $this->branch_id,
                        'bank_account_id' => $request->bank_account,
                    ]);
                }
            }
            return response()->json(['success'=>true,'message'=>'Speed points updated successfully!','url'=> route('admin.speed-points.index')]);
        }
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return Symfony\Component\HttpFoundation\Response
     */
    public function destroy(SpeedPoint $speedPoint)
    {
        abort_if(Gate::denies('speed_point_delete'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $speedPoint->forceDelete();
        toastr()->error('Speed point has been deleted!');
        return back();
    }

    /**
     * massDestroy for
     * destroy multiple
     * Speed Point using ids;
     *
     * @param  mixed $request
     * @return void
     */
    public function massDestroy(MassDestroySpeedPointRequest $request)
    {
        SpeedPoint::whereIn('id', request('ids'))->forceDelete();

        toastr()->error('Speed point has been deleted!');
        return response(null, Response::HTTP_NO_CONTENT);
    }
    public function delete(Request $request)
    {
        SpeedPoint::where('branch_id',$this->branch_id)->where('reconcilation_type',$request->machine)->where('date',$request->date)->where('batch_no',$request->batch)->forceDelete();
        return response('1',200);

    }
    public function update_bankstatement(Request $request)
    {
        abort_if(Gate::denies('speed_point_bank_statement'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $status = 1;
        $speed = SpeedPoint::query()->where('branch_id',$this->branch_id)->where('reconcilation_type',$request->machineid)->where('batch_no',$request->batchid);
        if($request->paymentid == null){
            $status = '0';
        }
        else{
            $exist = SpeedPoint::where('branch_id',$this->branch_id)->where('bank_statement_id',$request->paymentid)->where('batch_no',"!=",$request->batchid)->first();
            if(!empty($exist)){
                return response()->json(['status'=> 'error','message' => 'Payment is already assigned to other speedpoint']);
            }
        }
        $speed->update([
            'bank_statement_id' => $request->paymentid,
            'status' => $status,
        ]);
        $statement = BankStatement::where('id',$request->paymentid)->update(['status' => $status]);
        return response()->json('1',200);
    }

    // Speed Point Import
    public function speedPointImport(Request $request)
    {

        if($request->file('upload_file')->extension() == 'xlsx' || $request->file('upload_file')->extension() == 'csv' )
        {
            Excel::import(new SpeedPointImport,$request->file('upload_file'));
            toastr()->success("File Imported Successfully!");
            return back();
        }
        toastr()->error('File type is not correct!');
        return back();
    }
}
