<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Http\Requests\MassDestroyVatRequest;
use App\Http\Requests\StoreVatRequest;
use App\Http\Requests\UpdateVatRequest;
use App\Models\Vat;
use Illuminate\Http\Request;
use Symfony\Component\HttpFoundation\Response;
use Gate;
use Yajra\DataTables\Facades\DataTables;

class VatController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        abort_if(Gate::denies('vat_access'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        if ($request->ajax()) {
            $query = Vat::all();
            $table = DataTables::of($query);
            $table->addIndexColumn();

            $table->addColumn('DT_RowId',function($row){
                return $row->id ? $row->id :'';
            });
            $table->addColumn('placeholder', '&nbsp;');
            $table->addColumn('actions', '&nbsp;');
            $table->editColumn('status', function ($row) {
                 return $row->status == 1 ? '<span class="badge bg-inverse-success">Active</span>' : '<span class="badge bg-inverse-danger">Inactive</span>';
            });
            $table->editColumn('effective_from', function ($row) {
                return $row->effective_from ? date('d/m/Y',strtotime($row->effective_from)) : '';
            });

            $table->editColumn('effective_to', function ($row) {
                return $row->effective_to ? date('d/m/Y',strtotime($row->effective_to)) : '';
            });
            $table->editColumn('actions', function ($row) {
                $viewGate = 'vat_show';
                $editGate = 'vat_edit';
                $deleteGate = 'vat_delete';

                $crudRoutePart = 'vats';

                return view('partials.datatablesActions', compact(
                'viewGate',
                'editGate',
                'deleteGate',
                'crudRoutePart',
                'row'
            ));
            });

            $table->rawColumns(['actions', 'placeholder', 'header','status']);

            return $table->make(true);
        }

        return view('admin.vats.index');
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        abort_if(Gate::denies('vat_create'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        return view('admin.vats.create');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(StoreVatRequest $request)
    {
        $status = 0;

        // change d/m/Y to Y-m-d fromat
        $date1 = strtr($request->input('effective_from'), '/', '-');
        $eff_from = date('Y-m-d', strtotime($date1));

        if ($request->has('status')) {
            // Update effective_to and status
            $last_active = Vat::where('title',$request->title)
                ->where('status',1)
                ->first();
            if ($last_active != null) {
                $last_active->update([
                    'effective_to' => $eff_from,
                    'status' => $status,
                ]);
            }
            $status = $request->status;
        }

        Vat::create([
            'title' => $request->title,
            'value' => $request->value,
            'effective_from' => $eff_from,
            'remarks' => $request->remarks,
            'status' => $status,
        ]);

        toastr()->success('Vat has been created!');
        return redirect()->route('admin.vats.index');
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show(Vat $vat)
    {
        abort_if(Gate::denies('vat_show'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        return view('admin.vats.show',compact('vat'));
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit(Vat $vat)
    {
        abort_if(Gate::denies('vat_edit'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        return view('admin.vats.edit',compact('vat'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(UpdateVatRequest $request, Vat $vat)
    {
        $status = 0;
        // change d/m/Y to Y-m-d fromat
        $date1 = strtr($request->input('effective_from'), '/', '-');
        $eff_from = date('Y-m-d', strtotime($date1));

        if ($request->has('status')) {
            // Update effective_to and status
            $last_active = Vat::where('title',$request->title)
                ->where('status',1)
                ->first();
            if ($last_active != null) {
                $last_active->update([
                    'effective_to' => $eff_from,
                    'status' => $status,
                ]);
            }
            $status = $request->status;
        }
        if($vat->value != $request->value && (isset($vat->cashExpense) && count($vat->cashExpense) > 0) || (isset($vat->purchases) && count($vat->purchases) > 0)){
            toastr()->error('You cant update vat value as it has associated data!');
            return back();
        }
        $vat->update([
            'title' => $request->title,
            'value' => $request->value,
            'effective_from' => $eff_from,
            'remarks' => $request->remarks,
            'status' => $status,
        ]);

        toastr()->info('Vat has been updated!');
        return redirect()->route('admin.vats.index');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy(Vat $vat)
    {
        abort_if(Gate::denies('vat_delete'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        if(isset($vat->cashExpense) && count($vat->cashExpense) > 0 || (isset($vat->purchases) && count($vat->purchases) > 0)){
            toastr()->error('Vat has associated data. Please delete that first!');
            return back();
        }
        $vat->delete();
        toastr()->error('Vat has been deleted!');
        return back();
    }

    public function massDestroy(Request $request)
    {
        $request->validate([
            'ids'   => 'required|array',
        ]);
        foreach($request->ids as $id){
            $vat = Vat::find($id);
            $this->destroy($vat);
        }

        toastr()->error('Vat has been deleted!');
        return response(null, Response::HTTP_NO_CONTENT);
    }
}
